package net.lecnam.info.remotelist;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.lecnam.info.channel.ObjectReader;
import net.lecnam.info.channel.ObjectWriter;

import static net.lecnam.info.remotelist.RemoteList.logger;

public class Client implements Runnable {

	RemoteList<String> l;
	List<String> args;

	Client(ObjectReader<Answer<String>> in, ObjectWriter<Request<String>> out, List<String> args) {
		this.l = new RemoteList<>(in, out);
		this.args = args;
	}

	public boolean handle(List<String> args) {
		boolean done = false;
		switch (args.get(0)) {
			case "-size" -> {
				System.out.println("Size: " + l.size());
			}
			case "-get" -> {
				int index = Integer.parseInt(args.get(1));
				System.out.println("Value: " + l.get(index));
			}
			case "-set" -> {
				int index = Integer.parseInt(args.get(1));
				String value = args.get(2);
				l.set(index, value);
				System.out.println("Done.");
			}
			case "-add" -> {
				l.add(args.get(1));
				System.out.println("Done.");
			}
			case "-display" -> {
				System.out.println(l);
			}
			case "-debug" -> {
				logger.setLevel(Level.ALL);
				System.out.println(l);
			}
			case "-quit" -> {
				done = true;
			}
			default -> {
				System.out.println("Error: invalid command");
			}
		}
		return done;
	}

	@Override
	public void run() {
		BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
		if (args.isEmpty()) {
			System.out.println("Usage:");
			System.out.println("  -size : returns the size of the list.");
			System.out.println("  -add value : adds the value at the end of the list.");
			System.out.println("  -get index : returns the value at the given index.");
			System.out.println("  -set index value : sets the given value at the given index.");
			System.out.println("  -display : displays the content of the list.");
			System.out.println("  -debug : displays the content of the list with debug info.");
			System.out.println("  -i : switch to interactive mode.");
		} else {
			switch (args.get(0)) {
				case "-i" -> {
					try {
						boolean done = false;
						while (!done) {
							System.out.println("Command:");
							String line = reader.readLine();
							if (!line.isBlank()) {
								List<String> command = List.of(("-" + line).split("\\s"));
								//System.out.println(args);
								done = handle(command);
							}
						}
					} catch (IOException ex) {
						throw new Error(ex);
					}
				}
				default -> {
					handle(args);
				}
			}
		}
		l.stop();
	}

}
