package net.lecnam.info.remotelist;

import java.io.IOException;
import java.util.AbstractList;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.lecnam.info.channel.ObjectReader;
import net.lecnam.info.channel.ObjectWriter;
import net.lecnam.info.remotelist.Answer.*;
import net.lecnam.info.remotelist.Request.*;

public class RemoteList<E> extends AbstractList<E> {

	ObjectReader<Answer<E>> in;
	ObjectWriter<Request<E>> out;

	static final Logger logger = Logger.getLogger("client");
	
	RemoteList(ObjectReader<Answer<E>> in, ObjectWriter<Request<E>> out) {
		this.in = in;
		this.out = out;
		logger.setLevel(Level.OFF);
	}


	@Override
	public int size() {
		try {
			Request<E> request = new GetSize<E>();
			out.writeObject(request);
			Answer<E> answer = in.readObject();
			logger.info("Answer: " + answer);
			switch (answer) {
				case Size(var s) -> {
					return s;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (IOException ex) {
			throw new Error(ex);
		}
	}

	@Override
	public E get(int index) {
		try {
			Request<E> request = new GetValue<E>(index);
			out.writeObject(request);
			Answer<E> answer = in.readObject();
			logger.info("Answer: " + answer);
			switch (answer) {
				case Value(var elt) -> {
					return elt;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (IOException ex) {
			throw new Error(ex);
		}
	}

	@Override
	public E set(int index, E element) {
		try {
			Request<E> request = new Set<E>(index, element);
			out.writeObject(request);
			Answer<E> answer = in.readObject();
			logger.info("Answer: " + answer);
			switch (answer) {
				case OldValue(var elt) -> {
					return elt;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (IOException ex) {
			throw new Error(ex);
		}
	}

	@Override
	public E remove(int index) {
		try {
			Request<E> request = new Remove<E>(index);
			out.writeObject(request);
			Answer<E> answer = in.readObject();
			logger.info("Answer: " + answer);
			switch (answer) {
				case OldValue(var elt) -> {
					return elt;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (IOException ex) {
			throw new Error(ex);
		}
	}

	@Override
	public void add(int index, E element) {
		try {
			Request<E> request = new Add<>(index, element);
			out.writeObject(request);
			Answer<E> answer = in.readObject();
			logger.info("Answer: " + answer);
			switch (answer) {
				case Done() -> {
					return;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (IOException ex) {
			throw new Error(ex);
		}
	}

	public void stop() {
		try {
			Request<E> request = new Stop<>();
			out.writeObject(request);
		} catch (IOException ex) {
			throw new Error(ex);
		}
	}

}
